%\BeginDoc
%
%\Name: cycles 
%
%\Description: 
%     Exhaustive computation of cycles of the matroid
%     defined by the null space of a stoichiometric matrix S.
%
%\Author
%     Daniel A. Beard
%     Feng Yang
%     Biotech. & Bioengineering Center 
%     Medical College of Wisconsin, Milwaukee, Wisconsin
%
%\Remarks
%     1. Version1.1 was created at Jan 2, 2005.
%
%\References:
%  1. Yang F, Qian H, Beard DA: Ab Initio Prediction of Thermodynamically 
%     Feasible Reaction Directions from Biochemical Network Stoichiometry.
%     Metabolic Engineering, in review.
%  2. Beard DA, Liang SD, Qian H: Energy balance for analysis of complex 
%     metabolic networks. Biophys J 2002, 83:79-86.
%  3. Qian H, Beard DA, Liang SD: Stoichiometric network theory for 
%     nonequilibrium bio-chemical systems. Eur J Biochem 2003, 270:415-421.
%  4. Beard DA, Babson E, Curtis E, Qian H: Thermodynamic constraints for 
%     biochemical net-works. J Theor Biol 2004, 228:327-333.
%
%\Arguments
%   S          Stoichiometric Matrix. 
%
%\Program outputs:
%   N          Contains the cycle matrix of S.  
%              
%\EndDoc
%
%-------------------------------------------------------------------------    
%
    function [N] = cycles(S)  
%    
%   %----------------%
%   | Initialization |
%   %----------------%
%                         
    N = null(S,'r');                 % Compute a basis for null space of S.
    [m,n] = size(N);                 % initial size of N.   
 
    istop = 0;
    i = 1;
    n1 = 1;
    while (i <= n) & (istop ~= 1)
      [i n]
      Nnew = [];
      for k = (i-1)+1:i
        Nnew = [Nnew cycle_i(N,k)];
      end   

      %If Nnew is updated, test it before append into N.  
      if size(Nnew,2) ~= 0
        Nnew = test_new_vectors(Nnew,Nnew,2);
      end
            
      % append into N.
      N = [N Nnew];
      n = length(N(1,:));
      i = i + 1;
    end  %end all i 
    
    %Stop criteria.
    if  (i-1) == n & (i-1)+n1-1 >= n
      istop = 1;
    end    

    %Before exiting check for minimality.
    N = test_new_vectors(N,N,2);
%    
%  %------------------------%
%  | End of cycle function  |
%  %------------------------%
%   
%\BeginDoc
%
%\Name: cycle_i
%
%\Description: 
%   This code finds all new cycles based on i-th columns.
%
%\Arguments
%   N         current cycle matrix.
%   i         point to i-th column of N.
%
%\Program outputs:
%   Nnew       Contains new cycles.  
%              
%\EndDoc
%
%-------------------------------------------------------------------------    
%   
    function [Nnew] = cycle_i(N,i)
%    
%   %----------------%
%   | Initialization |
%   %----------------%
%
    n = length(N(1,:));
    Nnew = [];

    j = i+1;  
    while j <= n
%    
%     %----------------------------------------------------------%
%     | Compute new null space vectors from linear combinations  |
%     | of i^th and j^th columns of N. The set of vectors N is   |
%     | appended with the new minimal entries:                   |
%     %----------------------------------------------------------%
%          
      Nnew = [Nnew linear_combos(N, N(:,i), N(:,j) )];                          
      j = j+1;    
    end             
%    
%     %--------------------------------------------------------------%
%     | Test new null space vectors (Nnew) to see if they form       |
%     | cycles that are minimal against each other and vectors in N. | 
%     %--------------------------------------------------------------%
%       
    Nnew = test_new_vectors(Nnew,Nnew,2);   
    Nnew = test_new_vectors(N,Nnew,1);         
%    
%  %-------------------------------%
%  | End of cycle_dist_i function  |
%  %-------------------------------%
%
%\BeginDoc
%
%\Name: linear_combos 
%
%\Description: 
%   This code make a linear combination of two vectors. 
%
%\Arguments
%   v1,v2      Vectors with dimention n. 
%
%\Program outputs:
%   Nnew       Contains linear combinations of v1 and v2.  
%              
%\EndDoc
%
%-------------------------------------------------------------------------    
%        
    function [Nnew] = linear_combos(N,v1,v2);  
%    
%   %--------------------------------------------------------------------%
%   | Make sure v1 and v2 are not the proper subsets of v1+v2 or v1-v2.  |
%   %--------------------------------------------------------------------%
%      
    Nnew = [];  
    Nnew = [Nnew (v1+v2) (v1-v2)]; 
%    
%   %---------------------------------------------------------------------%
%   | Construct new linear combination so that its i-th entry be zero.    |
%   | Also gaurantee v1 and v2 are not the proper subsets of this vector. |
%   %---------------------------------------------------------------------%
%     
    n = length(v1);
    for i = 1:n
      v1i = v1(i);
      v2i = v2(i);
      if (v1i ~= 0) & (v2i ~= 0) & ( (abs(v1i)~=1)|(abs(v2i)~=1) )   
        Nnew = [Nnew (v2i.*v1 - v1i.*v2)];
      end            
    end

%   Safeguard wrt numerical error.     
    epsilon = 1e-10;
    Nnew = Nnew.*( abs(Nnew) > epsilon );
    
    Nnew = test_new_vectors(Nnew,Nnew,2);  % 2 means within the same matrix
    Nnew = test_new_vectors(N,Nnew,1);
%    
%  %------------------------------------%
%  | End of linear_combos function      |
%  %------------------------------------%
%
%\BeginDoc
%
%\Name: test_new_vectors 
%
%\Description: 
%    Function appends the set of column vectors
%    N with members in the set Nnew that are minimal(*) to one
%    another and minimal(*) to other vectors in N.
%    (*) 'minimal' in the sign-vector sense.
%
%\Arguments
%   N, Nnew     Nnew to be tested with respect to N/Nnew. 
%   option=1    Nnew to be tested with respect to N.
%   option=2    Nnew to be tested with respect to itself.
%
%\Program outputs:
%   Nadd        Filtered column vectors of Nnew were appended into N.        
%              
%\EndDoc
%
%-------------------------------------------------------------------------    
%        
    function [Nadd] = test_new_vectors(N,Nnew,option)
%    
%  %------------------------------------%
%  | Change to binary vector.           |
%  %------------------------------------%
%       
    Nadd = []; 
    n = size(N,2);
    nnew = size(Nnew,2);  
   
    if n == 0 || nnew == 0
      return
    end

    Ns = (N~=0);    
    Nsnew = abs(sign(Nnew));      
%    
%  %-------------------------------------------------%
%  | Each cycle should has minimum signed supports.  |
%  %-------------------------------------------------%
%    
    for j = 1:nnew  
      if option == 1      %   if N =/= Nnew
        t = Ns'*Nsnew(:,j) - sum(Ns,1)';                             
      else                %   if N = Nnew
        t = Ns(:,j+1:n)'*Nsnew(:,j) - sum(Ns(:,j+1:n),1)';                             
      end
      if sum(t~=0) == length(t) 
        Nadd = [Nadd Nnew(:,j)];
      end 
    end
%    
%  %---------------------------------------%
%  | End of test_new_vectors function      |
%  %---------------------------------------%
% 