%\BeginDoc
%     program driver_scripts
%
%     This example program is intended to illustrate few simple
%     examples of predicting flux direction and its preliminary analysis
%     in considerable detail.  This code may be used to understand basic 
%     idea of how to combine flux balance analysis and energy balance
%     analysis, as well as how to ab initio predict reaction 
%     directions for a biochemistry network.
%
%     Note that the computed steady-state flux directions satisfy
%     both mass balance constraints and thermodynamically 
%     feasible constraints.   
%
%\Author
%     Daniel A. Beard
%     Feng Yang
%     Biotech. & Bioengineering Center 
%     Medical College of Wisconsin, Milwaukee, Wisconsin
%
%\Remarks
%     1. Version1.1 was created at Jan 2, 2005.
%     2. These codes are freely distributed. However, we request that users  
%        notify Dr. Beard when using these codes. It is appropriate to cite 
%        Reference #1 in the following list in the publicatins using this
%        material.
%
%\References:
%  1. Yang F, Qian H, Beard DA: Ab Initio Prediction of Thermodynamically 
%     Feasible Reaction Directions from Biochemical Network Stoichiometry.
%     Metabolic Engineering, in review.
%  2. Beard DA, Liang SD, Qian H: Energy balance for analysis of complex 
%     metabolic networks. Biophys J 2002, 83:79-86.
%  3. Qian H, Beard DA, Liang SD: Stoichiometric network theory for 
%     nonequilibrium bio-chemical systems. Eur J Biochem 2003, 270:415-421.
%  4. Beard DA, Babson E, Curtis E, Qian H: Thermodynamic constraints for 
%     biochemical net-works. J Theor Biol 2004, 228:327-333.
%  5. Ilie Fishtik, etc: Reaction Route Graphs. I. Theory and Algorithm. J.
%     Phys. Chem. B 2004, 108, 5671-5682.
%
%\Examples       This driver has included seven different problems with
%                variety of difficulty. In detail,
%          =     1: A very simple example from Ref 4, Jan. 2004. 
%          =     2: Enlarged energy metablolism, see Ref 1.
%\EndDoc 
%
%-----------------------------------------------------------------------
%
%\User provided: 
%   S          Stoichiometric matrix; containts the stoichiometric numbers
%              for reactions in a reaction network.
%
%   ext_flux   Integer vector, indicates the external flux directions.
%              Positive direction was denoted as '+1' and negative as '-1'.
%              If there is no flux, set it to '0'.
%
%\Program outputs:
%   flux_dir   Vector stored predicted flux direction. 
% 
%-----------------------------------------------------------------------
%
%   %-----------------------------------------------------------%
%   | Clear variables and functions from memory and then select |
%   | example and the method used to predict flux direction.    |
%   |   if method = 1, using optimzation method.                |
%   |      method = 2, using enumeration method.                |
%   %-----------------------------------------------------------%
% 
clear;
example = 2;
method  = 2;   
%  
%   %-----------------------------------------------------------%
%   |                                                           |
%   |   L i s t   o f   e x a m p l e   n e t w o r k s         |
%   |                                                           |
%   %-----------------------------------------------------------%
% 
%
% Example 1 from Beard DA, Babson E, Curtis E, Qian H, see reference 4:
%
%   Expecting to find 13 cyles in matrix S, which represents a 
%   8-reaction network of 4 species:
% 
%       B ---r3---D---r7--- 
%       |  \      |
%       r2  r1    r4
%       |      \  |
% --r6- A ---r5---C---r8--- 
%  
    if example == 1;
    %        s1  s2  s3  s4  s5  s6  s7  s8    
        S = [ 0  -1   0   0  -1  +1   0   0
             -1  +1  -1   0   0   0   0   0
             +1   0   0  +1  +1   0   0  -1
              0   0  +1  -1   0   0  -1   0 ];   
        ext_flux = [1 1 1 ]; 
        C = cycles(S);         
    end   
      
% Example 2 from Yang F, Qian H, Beard DA, see reference 1.    
    if example == 2;
       inputfile = 'Ecoli_more.bcn';
       [S, C, species, enzymes] = read_data(inputfile);     
       ext_flux = [ 1 1 1 1 ];    
    end             
%  
%   %-----------------------------------------------------------%
%   |                                                           |
%   |               M A I N  F U N C T I O N                    |
%   |                                                           |
%   %-----------------------------------------------------------%
%      
%   %------------------------------------------%
%   | Step 1. Extract basis from Cycle matrix  |
%   %------------------------------------------%
%     
    [m,n_all] = size(S);                 % m is the number of reactants
                                         % n_all is total number of fluxes                                         
    n_ext = length(ext_flux);            % the number of external fluxes. 
    n_int = n_all - n_ext;               % the number of internal fluxes.
       
    [C_in C_ex] = extract_basis(C,ext_flux);    
    nc_ex = size(C_ex,2);                % number of columns of external cycle. 
    nc_in = size(C_in,2);                % number of columns of internal cycle.    
    flux_dir = zeros(n_int,1);    
%  
%   %------------------------------------------%
%   | Step 2A: Optimization method             |
%   %------------------------------------------%
%     
    if method == 1                         
        LB = zeros(nc_ex,1); 
        UB = +10.*ones(nc_ex,1);  %UB is an array of arbitrary finite 
                                  %positive real numbers. 

        % A feasible initial guess for calculations below:
        xo = ones(nc_ex,1); 
        for i = 1:n_int 
             max_min= -1;
             [x,F,xflag(i,1)] = fmincon(@flux,xo,-C_ex(n_int+1:n_all,:),... 
                 zeros(n_ext,1),[],[],LB,UB,@T_constraint,[],C_in,C_ex,max_min,i);         
             J_min(i) = F;  

             max_min= 1;     
             [x,F,xflag(i,2)] = fmincon(@flux,xo,-C_ex(n_int+1:n_all,:),... 
                 zeros(n_ext,1),[],[],LB,UB,@T_constraint,[],C_in,C_ex,max_min,i);         
             J_max(i) = -F; 
        end 
        [(1:n_int)' J_min' J_max' ];        
        
        % '-11' is denoted as reversible flux, while '-1' as negative 
        % and '+1' as positive flux.
        for i = 1:n_int
            if abs(J_min(i)) < 1e-8
                J_min(i) = 0;
            end
            if abs(J_max(i)) < 1e-8
                J_max(i) = 0;
            end            
            
            if J_min(i)*J_max(i) < 0 
                flux_dir(i) = -11;
            elseif J_max(i) == 0 & J_min(i) < 0 
                flux_dir(i) = -1;
            elseif J_max(i) > 0 & J_min(i) == 0 
                flux_dir(i) = 1;
            end
        end
                
        % Optimization process may fail.
        if sum(sum( xflag<0)) ~= 0 
             disp('error in optimization'); 
             xflag; 
        end         
    end  
%  
%   %------------------------------------------%
%   | Step 2B: Enumeration method              |
%   %------------------------------------------%
%
    if method == 2 
        flux_dir = sign_pattern(S, C_ex, ext_flux);             
    end
%    
    disp(['-------------------------------------------------']);
    disp([' Reaction direction prediction']);
    disp([' (+) = 1,      (-) = -1,     (+)(-) = -11']);         
    disp(['-------------------------------------------------']);    
    disp(['  index flux direction']);         
    disp([(1:n_int)'   flux_dir(1:n_int)])    
%  
%   %------------------------------------------%
%   | Step 3: Check thermodynamic feasibility  |
%   %------------------------------------------%
%
    k = 1;
    match = [];
    for j = 1:nc_in
        for i = 1:n_all
            if C_in(i,j) ~= 0 &&  flux_dir(i) ~= -11 
                 if C_in(i,j)*flux_dir(i) < 0
                    match(k) = 1;
                 else
                    match(k) = -1;
                 end
                 k = k + 1;
            end
        end
        if length(match) > 0 && abs(sum(match)) == sum(ones(length(match),1))
            cycle(j) = 1;
        else
            cycle(j) = -1;
        end        
    end

    disp(['-------------------------------------------------']);
    disp([' Any infeasible cycle included in the sign pattern? ']);
    disp(['   yes = 1,  no = -1' ]);         
    disp(['-------------------------------------------------']);    
    disp(['  index yes/no']);  
    disp([(1:nc_in)'  cycle']);    
  
